unit testproc;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Tlhelp32, StdCtrls, Menus, Buttons;

const
  PROCESS_TERMINATE = $0001;            // const for killing a proc
  PROCESS_SET_INFORMATION   = $0200;    // const for changing priority
// (should be in windows.pas if you have it)
type
  TForm1 = class(TForm)
    PopupMenu1: TPopupMenu;
    mnuRefresh: TMenuItem;
    GroupBox1: TGroupBox;
    Flagslbl: TLabel;
    ucountlbl: TLabel;
    procidlbl: TLabel;
    heaplbl: TLabel;
    modlbl: TLabel;
    threadlbl: TLabel;
    parentlbl: TLabel;
    priolbl: TLabel;
    TheFlags: TLabel;
    Thecount: TLabel;
    TheProcID: TLabel;
    TheHeapId: TLabel;
    TheModID: TLabel;
    TheThreads: TLabel;
    Theparent: TLabel;
    CloseBtn: TBitBtn;
    RefreshBtn: TBitBtn;
    KillBtn: TBitBtn;
    Label1: TLabel;
    ListBox1: TListBox;
    mnuKill: TMenuItem;
    ThePrio: TComboBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure RefreshActionExecute(Sender: TObject);
    procedure KillActionExecute(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure ListBox1KeyPress(Sender: TObject; var Key: Char);
    procedure ThePrioChange(Sender: TObject);
  private
    procedure UpdateLabels;             // sync the text list box with the internal list
    procedure LookForExes;              // update the kist box with the executable names
    procedure EmptyListbox;             // clear the text list box
    { Private declarations }
  public
    { Public declarations }
  end;
(********************************************************************************************)
  procObject = class                    // class for use with Tlistbox.objects
  private
    fProc: TProcessEntry32;             // declare all data private for any future changes
    procedure putProcData(const value: TProcessEntry32);
    function getProcData: TProcessEntry32;
    procedure setPriority(const value: integer);
    function getPriority: Integer;
  public
    constructor create(procinfo: TProcessEntry32);
  published
    property Theproc: TProcessEntry32 read getProcData write putProcData;
    property ProcPriority: Integer read getPriority write setPriority;
  end;
(********************************************************************************************)
var
  Form1: TForm1;

implementation

{$R *.DFM}

var
  killEnabled: boolean = false;
(********************************************************************************************)

constructor procObject.create(procinfo: TProcessEntry32);
begin
  inherited create;                     // need to specify which inherited constructor to call
  theproc := procinfo;                  // since parameter list differ
end;
(********************************************************************************************)

procedure procObject.putProcData(const value: TProcessEntry32);
begin                                   // setter for property Theproc
  fproc := value;
end;
(********************************************************************************************)

function procObject.getProcData: TProcessEntry32;
begin                                   // getter for property Theproc
  result := fproc;
end;
(********************************************************************************************)

procedure procObject.setPriority(const value: Integer);
var  ProcHandle: THandle;
begin                                   // setter for property Theproc
  ProcHandle := OpenProcess(PROCESS_SET_INFORMATION, false, fProc.th32ProcessID);
  if not SetPriorityClass(ProcHandle, value) then
    ShowMessage('An error occured while trying to change process priority')
  else
      fProc.pcPriClassBase := value;
  if prochandle > 0 then
    CloseHandle(ProcHandle);
end;
(********************************************************************************************)

function procObject.getPriority: integer;
begin                                   // getter for property Theproc
  result := fProc.pcPriClassBase;
end;
(********************************************************************************************)

procedure TForm1.updatelabels;          // synch the UI with the internal process list
begin
  if listbox1.itemindex <> -1 then
    if assigned(listbox1.items.objects[listbox1.itemindex]) then
      with procObject(listbox1.items.objects[listbox1.itemindex]) do
        begin
          with TheProc do
            begin
              theflags.caption := inttostr(dwFlags);
              thecount.caption := inttostr(cntUsage);
              theprocid.caption := inttostr(th32ProcessID);
              theheapid.caption := inttostr(th32DefaultHeapID);
              themodid.caption := inttostr(th32DefaultHeapID);
              thethreads.caption := inttostr(cntthreads);
              theparent.caption := inttostr(th32ParentProcessID);
            end;
          with ThePrio do
            case procpriority of
              4: ItemIndex := 0;        // the only possible priority values
              8: ItemIndex := 1;        // are 4, 8, 13, and 24
              13: ItemIndex := 2;
              24: ItemIndex := 3;
            end;
          // only enable process kills if something is selected
          KillEnabled := true;
        end
end;
(********************************************************************************************)

procedure Tform1.EmptyListbox;          // frees the list and associated objects
var
  x: integer;
  allitems: integer;
begin
  allitems := listbox1.Items.count;
  if allitems > 0 then
    for x := allitems - 1 downto 0 do
      if assigned(listbox1.items.objects[x]) then
        listbox1.items.objects[x].free;
  listbox1.clear;
end;
(********************************************************************************************)

procedure Tform1.LookForExes;
var
  Proc: TProcessEntry32;
  Snap: THandle;
  prochandle: procObject;
  tempname: string;
  oldindex: integer;
begin
  oldindex := listbox1.ItemIndex;       // save old listbox position
  if listbox1.items.count - 1 < oldindex then // check to see if any processes have been killed
    dec(oldindex);                      // and update the index accordingly
  EmptyListbox;                         // clear UI list
  try
    listbox1.items.BeginUpdate;         // disable screen refreshes during update
    listbox1.items.clear;
    try
      // API calls. Picked from someone on the Borland NG's. Sorry ... forgot the name.
      Snap := CreateToolHelp32SnapShot(TH32CS_SNAPALL, 0); // API in ToolHelp.pas (dcu)
      Proc.dwSize := SizeOf(TProcessEntry32);
      Process32First(Snap, Proc);
      repeat
        prochandle := procObject.create(proc);
        if length(string(Proc.szExeFile)) > 0 then // check for processes without a description
          tempname := proc.szExeFile
        else
          tempname := '< Unknown >';
        listbox1.items.addobject(tempname, prochandle); // save the process object to the list
      until (not Process32Next(Snap, Proc));
    except
      showmessage('An error occurred while reading task information');
    end;
  finally
    if listbox1.Items.Count > 0 then    // set listbox position to old position - 1
      begin
        if oldindex > 0 then
          listbox1.ItemIndex := oldindex
        else
          listbox1.ItemIndex := 0;
      end
    else
      KillEnabled := false;
    listbox1.items.endupdate;           // reenable screen updates
  end;
end;
(********************************************************************************************)

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  EmptyListbox;
end;
(********************************************************************************************)

procedure TForm1.RefreshActionExecute(Sender: TObject);
begin
  Lookforexes;                          // update the object list
  updatelabels;
end;
(********************************************************************************************)

procedure TForm1.KillActionExecute(Sender: TObject);
var
  proc: procobject;
  procHandle: THandle;
  ExitCD: integer;
// This does a "hard" kill. From the MS help file :
// The TerminateProcess function is used to unconditionally cause a process to exit.
// Use it only in extreme circumstances. The state of global data maintained by dynamic-link
// Libraries (DLLs) may be compromised if TerminateProcess is used rather than ExitProcess.
// TerminateProcess causes all threads within a process to terminate, and causes a process
// to exit, but DLLs attached to the process are not notified that the process is terminating.
begin
  if killEnabled then
    begin
      screen.Cursor := crhourglass;
      ExitCD := 0;
      if listbox1.itemindex <> -1 then
        begin
          proc := procObject(listbox1.items.objects[listbox1.itemindex]); // get handle
          procHandle := 0;
          try                           // and kill process
            procHandle := OpenProcess(PROCESS_TERMINATE, false, Proc.Theproc.th32ProcessID);
            TerminateProcess(procHandle, ExitCd);
          except
            showmessage('An error has occurred while trying to kill the selected application.' +
              #13 + 'Error code :' + inttostr(ExitCD));
          end;
          if procHandle <> 0 then
            CloseHandle(procHandle);
        end;
      if ExitCD = 0 then
        begin
          // wait for windows to kill the task before refreshing the app list
          sleep(200);
          lookforexes;                  // refresh the process list
          screen.cursor := crDefault;
          updatelabels;                 // and display selected process information
        end;
    end;
end;
(********************************************************************************************)

procedure TForm1.ThePrioChange(Sender: TObject);
var
  Newpriority: Integer;
begin
  case ThePrio.ItemIndex of
    0: Newpriority := IDLE_PRIORITY_CLASS;
    2: Newpriority := HIGH_PRIORITY_CLASS;
    3: Newpriority := REALTIME_PRIORITY_CLASS;
    else
      Newpriority := NORMAL_PRIORITY_CLASS; // normal priority by default
  end;
  procObject(listbox1.items.objects[listbox1.itemindex]).setPriority(Newpriority);
  lookforexes;
  updatelabels;
end;
(********************************************************************************************)

procedure TForm1.ListBox1Click(Sender: TObject);
begin
  updatelabels;                         // synch object information with description labels
end;
(********************************************************************************************)

procedure TForm1.CloseBtnClick(Sender: TObject);
begin
  self.close;
end;
(********************************************************************************************)

procedure TForm1.ListBox1KeyPress(Sender: TObject; var Key: Char);
begin
  updatelabels;
end;
(********************************************************************************************)

end.

